@@ -1,4 +1,8 @@
-"""Module for managing automation sequences (series of Steps)"""
+"""
+DEPRECATED: use the `.sequencing.automation` module instead.
+
+Module for managing automation sequences (series of Steps).
+"""
 import warnings
 from .sequencing.automation import Automation, build_automation_sequence
 
