@@ -77,6 +77,7 @@ class TemporaryCallbackSetter(ABC):
                  *components: 'ComponentState',
                  callbacks: Union[Callable, Iterable[Callable]] = None,
                  threshold_state: StateSet = OperationalState.OFFLINE,
+                 pre_state_change: bool = True,
                  ):
         """
         Context manager for temporarily applying callbacks to the provided components
@@ -84,9 +85,11 @@ class TemporaryCallbackSetter(ABC):
         :param components: components to monitor
         :param callbacks: custom callback(s) to apply
         :param threshold_state: threshold state to apply callbacks at and below
+        :param pre_state_change: whether the callbacks should be added to pre- or post-state change callback lists
         """
         self.components = components
         self.threshold_state = threshold_state
+        self.pre_state_change = pre_state_change
         if callbacks is None:
             callbacks = []
         elif callable(callbacks):  # cast if it's only a single callback
@@ -99,7 +102,7 @@ class TemporaryCallbackSetter(ABC):
         for component in self.components:
             for state in getattr(component, self._state_set_attribute):
                 if state <= self.threshold_state:
-                    state.register_callbacks(self.callback_capture, *self.callbacks)
+                    state.register_callbacks(self.callback_capture, *self.callbacks, pre_state=self.pre_state_change)
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -227,11 +230,18 @@ class ComponentState(InstanceRegistry):
         state = self.get_operational_state_from_key(state)
         # execute callbacks if any are specified
         try:
-            if state.has_callbacks:
-                self.logger.info(f'executing {len(state.value.callbacks)} callbacks for {state.name}')
-            out = state.execute_callbacks()
+            if state.has_pre_state_change_callbacks:
+                self.logger.info(
+                    f'executing {len(state.value.pre_state_callbacks)} pre-state-change callbacks for {state.name}'
+                )
+            out = state.execute_pre_state_callbacks()
         finally:
             self._current_operational_state = state
+        if state.has_post_state_change_callbacks:
+            self.logger.info(
+                f'executing {len(state.value.pre_state_callbacks)} post-state-change callbacks for {state.name}'
+            )
+            out.extend(state.execute_post_state_callbacks())
         return out
 
     @property
@@ -258,11 +268,18 @@ class ComponentState(InstanceRegistry):
             raise TypeError(f'the type {type(state)} is not supported for a current component action state')
         try:
             # execute callbacks if they are specified
-            if state.has_callbacks:
-                self.logger.info(f'executing {len(state.value.callbacks)} callbacks for {state.name}')
-            out = state.execute_callbacks()
+            if state.has_pre_state_change_callbacks:
+                self.logger.info(
+                    f'executing {len(state.value.pre_state_callbacks)} pre-state-change callbacks for {state.name}'
+                )
+            out = state.execute_pre_state_callbacks()
         finally:
             self._current_action_state = state
+        if state.has_post_state_change_callbacks:
+            self.logger.info(
+                f'executing {len(state.value.pre_state_callbacks)} post-state-change callbacks for {state.name}'
+            )
+            out.extend(state.execute_post_state_callbacks())
         return out
 
     @classmethod
