@@ -226,21 +226,31 @@ class TestStates(unittest.TestCase):
             return 'asdf'
 
         # test registration
-        self.test_state_one.register_callbacks(banana, melon)
-        self.assertEqual(len(self.test_state_one.callbacks), 2, "ensure callback was added")
-        self.assertTrue(banana in self.test_state_one.callbacks, 'check that method is registered')
+        self.test_state_one.register_callbacks(banana)
+        self.test_state_one.register_post_state_callbacks(melon)
+        self.assertEqual(len(self.test_state_one.pre_state_callbacks), 1, "ensure callback was added")
+        self.assertEqual(len(self.test_state_one.post_state_callbacks), 1, 'ensure post callback was added')
+        self.assertTrue(banana in self.test_state_one.pre_state_callbacks, 'check that method is registered')
         self.assertRaises(TypeError, self.test_state_one.register_callbacks, 1, 'check that non-callables are rejected')
 
         # test execution
-        values = self.test_state_one.execute_callbacks()
-        self.assertEqual(len(values), 2, 'check that two returns were captured')
+        values = self.test_state_one.execute_pre_state_callbacks()
+        self.assertEqual(len(values), 1, 'check that two returns were captured')
         self.assertEqual(values[0], 1234, 'check that first function value is correct')
-        self.assertEqual(values[1], 'asdf', 'check that second function value is correct')
+        post_values = self.test_state_one.execute_post_state_callbacks()
+        self.assertEqual(post_values[0], 'asdf', 'check that second function value is correct')
 
         # test deregistration
         self.test_state_one.deregister_callbacks(banana)
-        self.assertEqual(len(self.test_state_one.callbacks), 1, 'ensure callback was removed')
-        self.assertFalse(banana in self.test_state_one.callbacks, 'check that method was deregistered')
+        self.assertEqual(len(self.test_state_one.pre_state_callbacks), 0, 'ensure callback was removed')
+        self.assertFalse(self.test_state_one.has_pre_state_change_callbacks, 'ensure pre-state-change callback was removed')
+        self.assertTrue(self.test_state_one.has_post_state_change_callbacks, 'ensure post-state-change callback was unaffected')
+        self.assertFalse(banana in self.test_state_one.pre_state_callbacks, 'check that method was deregistered')
+
+        self.test_state_one.deregister_callbacks(melon)
+        self.assertFalse(self.test_state_one.has_callbacks)
+        self.assertFalse(self.test_state_one.has_post_state_change_callbacks)
+        self.assertFalse(self.test_state_one.has_pre_state_change_callbacks)
 
 
 class FancyOperationalStates(StateSet):
@@ -248,6 +258,7 @@ class FancyOperationalStates(StateSet):
     BAD = State(-1)
     OFFLINE = 0
     GOOD = State(1)
+    GREAT = State(2)
 
 
 class FancyTestComponent(ComponentState):
@@ -297,37 +308,93 @@ class TestComponentCallbacks(unittest.TestCase):
     @classmethod
     def setUpClass(cls) -> None:
         cls.component = FancyTestComponent()
+        cls.lock = threading.Lock()
 
     def test_registration(self):
         """ensure actions were registered as expected"""
-        self.assertTrue(self.component.critical_action in self.component.operational_states.BAD.value.callbacks)
-        self.assertTrue(self.component.restore_functionality in self.component.operational_states.GOOD.value.callbacks)
+        self.assertTrue(self.component.critical_action in self.component.operational_states.BAD.value.pre_state_callbacks)
+        self.assertTrue(self.component.restore_functionality in self.component.operational_states.GOOD.value.pre_state_callbacks)
 
     def test_callback_functionality(self):
         """tests callback functionality"""
-        try:
-            # change the operational state to a bad state; ensure callback was executed
-            value = self.component.update_operational_state(-1)
-            self.assertEqual(value[0], self.component.bad_string, 'ensure return value is captured')
-            self.assertTrue(self.component.real_bad_thing_happened, 'ensure callback executed as expected')
+        with self.lock:
+            try:
+                # change the operational state to a bad state; ensure callback was executed
+                value = self.component.update_operational_state(-1)
+                self.assertEqual(value[0], self.component.bad_string, 'ensure return value is captured')
+                self.assertTrue(self.component.real_bad_thing_happened, 'ensure callback executed as expected')
 
-            # change the operational state to a good one; ensure callback was executed
-            value = self.component.update_operational_state(1)
-            self.assertEqual(value[0], self.component.good_string, 'ensure return value is captured')
-            self.assertFalse(self.component.real_bad_thing_happened, 'ensure callback executed as expected')
+                # change the operational state to a good one; ensure callback was executed
+                value = self.component.update_operational_state(1)
+                self.assertEqual(value[0], self.component.good_string, 'ensure return value is captured')
+                self.assertFalse(self.component.real_bad_thing_happened, 'ensure callback executed as expected')
 
-            # test condition check within restor function
-            value = self.component.update_operational_state(1)
-            self.assertEqual(value[0], self.component.no_change, 'ensure conditional checks work')
+                # test condition check within restor function
+                value = self.component.update_operational_state(1)
+                self.assertEqual(value[0], self.component.no_change, 'ensure conditional checks work')
 
-            # these will fail if the state is updated before the callbacks are executed
-            value = self.component.update_operational_state(-9000)
-            self.assertEqual(value[0], self.component.allowable_jump)
-            value = self.component.update_action_state(1)
-            self.assertEqual(value[0], self.component.allowable_jump)
-        finally:
-            self.component.update_operational_state(1)
-            self.component.update_action_state(0)
+                # these will fail if the state is updated before the callbacks are executed
+                value = self.component.update_operational_state(-9000)
+                self.assertEqual(value[0], self.component.allowable_jump)
+                value = self.component.update_action_state(1)
+                self.assertEqual(value[0], self.component.allowable_jump)
+            finally:
+                self.component.update_operational_state(1)
+                self.component.update_action_state(0)
+
+    def pre_state_check_callback(self):
+        """special callback which should be executed before a component state is changed"""
+        self.assertEqual(self.component.current_operational_state, 1)
+
+    def post_state_check_callback(self):
+        """special callback which should be executed after a component state is changed"""
+        self.assertEqual(self.component.current_operational_state, 2)
+
+    def test_operational_callback_ordering(self):
+        """
+        tests that pre- and post- operational state change callbacks are executed in the correct place
+        """
+        with self.lock:
+            try:
+                self.component.update_operational_state(1)  # start from 1
+                state = self.component.operational_states(2)
+                state.register_pre_state_callbacks(self.pre_state_check_callback)
+                state.register_post_state_callbacks(self.post_state_check_callback)
+                self.component.update_operational_state(2)
+                # deregister and check
+                state.deregister_callbacks(self.pre_state_check_callback, self.post_state_check_callback)
+                self.assertFalse(self.pre_state_check_callback in state.value.pre_state_callbacks)
+                self.assertFalse(self.post_state_check_callback in state.value.post_state_callbacks)
+            finally:
+                self.component.update_operational_state(1)
+                self.component.update_action_state(0)
+
+    def pre_action_state_check_callback(self):
+        """special callback which should be executed before a component state is changed"""
+        self.assertEqual(self.component.current_action_state, 0)
+
+    def post_action_state_check_callback(self):
+        """special callback which should be executed after a component state is changed"""
+        self.assertEqual(self.component.current_action_state, 1)
+
+    def test_action_callback_ordering(self):
+        """
+        tests that pre- and post- operational state change callbacks are executed in the correct place
+        """
+        with self.lock:
+            try:
+                self.component.update_action_state(0)
+                state = self.component.action_states(1)
+                state.register_pre_state_callbacks(self.pre_action_state_check_callback)
+                state.register_post_state_callbacks(self.post_action_state_check_callback)
+                self.component.update_action_state(1)
+                # deregister and check
+                state.deregister_callbacks(self.pre_action_state_check_callback, self.post_action_state_check_callback)
+                self.assertFalse(self.pre_action_state_check_callback in state.value.pre_state_callbacks)
+                self.assertFalse(self.post_action_state_check_callback in state.value.post_state_callbacks)
+            finally:
+                self.component.update_operational_state(1)
+                self.component.update_action_state(0)
 
 
 class SoManyStates(StateSet):
@@ -362,13 +429,13 @@ class TestTemporaryCallback(unittest.TestCase):
             if state <= threshold:
                 self.assertTrue(state.has_callbacks, f'ensure callback was assigned to {state}')
                 self.assertTrue(
-                    fn in state.value.callbacks,
+                    fn in state.value.pre_state_callbacks,
                     f'ensure correct callback was assigned to {state}'
                 )
             else:
                 self.assertFalse(state.has_callbacks, 'ensure callback was not assigned to states above threshold')
                 self.assertFalse(
-                    fn in state.value.callbacks,
+                    fn in state.value.pre_state_callbacks,
                     f'ensure callback was not assigned to {state}'
                 )
 
@@ -383,7 +450,7 @@ class TestTemporaryCallback(unittest.TestCase):
                     self.assertTrue(tcb.callback_encountered, 'ensure callback was encountered')
                 for state in self.component.operational_states:
                     self.assertFalse(
-                        tcb.callback_capture in state.value.callbacks,
+                        tcb.callback_capture in state.value.pre_state_callbacks,
                         'ensure callback was removed'
                     )
                     self.assertFalse(state.has_callbacks)
@@ -408,7 +475,7 @@ class TestTemporaryCallback(unittest.TestCase):
                 # ensure callback was unassigned after exiting
                 for state in self.component.operational_states:
                     self.assertFalse(state.has_callbacks)
-                    self.assertFalse(tcb.callback_capture in state.value.callbacks)
+                    self.assertFalse(tcb.callback_capture in state.value.pre_state_callbacks)
             finally:
                 self.component.update_operational_state(1)
 
@@ -429,12 +496,12 @@ class TestTemporaryCallback(unittest.TestCase):
                 if state <= threshold_state:
                     self.assertTrue(state.has_callbacks, f'ensure callback was assigned to {state}')
                     self.assertTrue(
-                        self.custom_callback in state.value.callbacks,
+                        self.custom_callback in state.value.pre_state_callbacks,
                         f'ensure correct callback was assigned to {state}'
                     )
                 elif hasattr(state.value, 'callbacks'):
                     self.assertFalse(state.has_callbacks, 'ensure callback was not assigned to states above threshold')
                     self.assertFalse(
-                        self.custom_callback in state.value.callbacks,
+                        self.custom_callback in state.value.pre_state_callbacks,
                         f'ensure callback was not assigned to {state}'
                     )
